# -*- coding: utf-8 -*-
"""
Created on Mon May  4 14:29:36 2020

@author: Jon Hvideberg Holte
"""
import pandas as pd
import numpy as np
import scipy.interpolate
import matplotlib.pyplot as plt




#WindData = np.loadtxt(fname = "S9_wind_speed_2012.txt")
WindData_df = pd.read_excel("wind_speed_raw.xlsx", header = None)       #This is the raw data
WindData = WindData_df[0].astype(float).tolist()    #Storing parameter list values from the dataframe and storing in dictionary


#Etrapolate the wind speeds to a spesific height
h_ref   = 82.6
h       = 60
alpha   = 1/7
WindData = [x * (h/h_ref)**alpha for x in WindData]


#power_curve = np.loadtxt(fname = "S10_power_curve_function.txt")
power_curve_df = pd.read_excel("power_curves.xlsx", sheet_name = "Enercon E-53" , header = None)       #This is the raw data
power_curve = power_curve_df.to_numpy()



windData_winter = WindData[0:1440]     #Jan, feb and des
windData_winter = np.append(windData_winter,WindData[8040:8784]) 
windData_spring = WindData[1440:3648]  #Mars, apr and may
windData_summer = WindData[3648:5856]     #Juni, july and aug
windData_autumn = WindData[5856:8040]     #Sep, oct and nov

def power_output (power_curve, wind_speeds):
    # The x-values in the power curve (wind speed [m/s])
    x = power_curve[:,0]
    # The y-values in the power curve (relative power output {0,1})
    y = power_curve[:,1]
    # Define the function that interpolate the wanted value
    y_interp = scipy.interpolate.interp1d(x, y)
    
    power_out = []   #Create a empty list
    
    for i in range(0,len(wind_speeds)):       #Iterate for all the generated wind speed values
        power_out.append(float(y_interp(wind_speeds[i]))) #For every wind speed assign a power output from the power curve and store in list

    return power_out

wind_power_winter = power_output(power_curve,windData_winter)
wind_power_spring = power_output(power_curve,windData_spring)
wind_power_summer = power_output(power_curve,windData_summer)
wind_power_autumn = power_output(power_curve,windData_autumn)


#Decide what the name of the output file should be
excel_file_r = "wind_data_new.xlsx"

# Create a Pandas Excel writer object using XlsxWriter as the engine. 
writer = pd.ExcelWriter(excel_file_r, engine ='xlsxwriter')

#For Storage values StoreV
#df_WindData_winter = pd.DataFrame(windData_winter)     #define the values in a dataframe
#df_WindData_spring = pd.DataFrame(windData_spring)
#df_WindData_summer = pd.DataFrame(windData_summer)
#df_WindData_autumn = pd.DataFrame(windData_autumn)

#df_WindData_winter.to_excel(writer, sheet_name ="winter", index=False)                      #write that dataframe to Excel
#df_WindData_spring.to_excel(writer, sheet_name ="spring", index=False, startcol=1) 
#df_WindData_summer.to_excel(writer, sheet_name ="summer", index=False, startcol=2) 
#df_WindData_autumn.to_excel(writer, sheet_name ="autumn", index=False, startcol=3) 

#Since the data in the array is a 3d-array, it must be reshaped into the dataframe
#It is reshaped into rows=day(NR) and columns=probability

for i in range (round(len(windData_winter)/24)):
    k = 24*i
    l = 24*i + 24
    #Store wind speed data:
    windData_winter_temp = windData_winter[k:l]
    df_WindData_winter_temp = pd.DataFrame(windData_winter_temp)     #define the values in a dataframe
    df_WindData_winter_temp.to_excel(writer, sheet_name ="winter_wind", index=False, startcol=i)

    #Store wind power output data:
    wind_power_winter_temp = wind_power_winter[k:l]
    df_wind_power_winter_temp = pd.DataFrame(wind_power_winter_temp)     #define the values in a dataframe
    df_wind_power_winter_temp.to_excel(writer, sheet_name ="winter_power", index=False, startcol=i)
    
for i in range (round(len(windData_spring)/24)):
    k = 24*i
    l = 24*i + 24
    #Store wind speed data:
    windData_spring_temp = windData_spring[k:l]
    df_windData_spring_temp = pd.DataFrame(windData_spring_temp)     #define the values in a dataframe
    df_windData_spring_temp.to_excel(writer, sheet_name ="spring_wind", index=False, startcol=i)
    
    #Store wind power output data:
    wind_power_spring_temp = wind_power_spring[k:l]
    df_wind_power_spring_temp = pd.DataFrame(wind_power_spring_temp)     #define the values in a dataframe
    df_wind_power_spring_temp.to_excel(writer, sheet_name ="spring_power", index=False, startcol=i)
    
for i in range (round(len(windData_summer)/24)):
    k = 24*i
    l = 24*i + 24
    #Store wind speed data:
    windData_summer_temp = windData_summer[k:l]
    df_windData_summer_temp = pd.DataFrame(windData_summer_temp)     #define the values in a dataframe
    df_windData_summer_temp.to_excel(writer, sheet_name ="summer_wind", index=False, startcol=i)

    #Store wind power output data:    
    wind_power_summer_temp = wind_power_summer[k:l]
    df_wind_power_summer_temp = pd.DataFrame(wind_power_summer_temp)     #define the values in a dataframe
    df_wind_power_summer_temp.to_excel(writer, sheet_name ="summer_power", index=False, startcol=i)
    
for i in range (round(len(windData_autumn)/24)):
    k = 24*i
    l = 24*i + 24
    #Store wind speed data:
    windData_autumn_temp = windData_autumn[k:l]
    df_windData_autumn_temp = pd.DataFrame(windData_autumn_temp)     #define the values in a dataframe
    df_windData_autumn_temp.to_excel(writer, sheet_name ="autumn_wind", index=False, startcol=i)
 
    #Store wind power output data:
    wind_power_autumn_temp = wind_power_autumn[k:l]
    df_wind_power_autumn_temp = pd.DataFrame(wind_power_autumn_temp)     #define the values in a dataframe
    df_wind_power_autumn_temp.to_excel(writer, sheet_name ="autumn_power", index=False, startcol=i)



# Plot the power curve
plt.figure(1)
plt.xlabel("Wind speed [m/s]") 
plt.ylabel("Power output [p.u.]")
plt.plot(power_curve[0:26,0],power_curve[0:26,1], "r-o", label="Power curve")
plt.legend()
plt.savefig('power_curve_Enercon.png')

# Close the Pandas Excel writer object and output the Excel file. 
writer.save()